import logging
import os

from pythonjsonlogger import jsonlogger

JSON_FORMAT = os.getenv(
    "JSON_FORMAT",
    "%(asctime)s %(levelname)s %(filename)s %(lineno)s %(name)s %(funcName)s %(message)s",
)
LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO")
ENABLE_JSON_LOGGING = os.getenv("ENABLE_JSON_LOGGING", "true")


class CustomJsonFormatter(jsonlogger.JsonFormatter):
    def add_fields(self, log_record, record, message_dict):
        super(CustomJsonFormatter, self).add_fields(log_record, record, message_dict)
        if log_record.get("level"):
            log_record["level"] = log_record["level"].upper()
        else:
            log_record["level"] = record.levelname


class LoggingMixin:
    enable_json_logging = ENABLE_JSON_LOGGING.lower()
    format = (
        JSON_FORMAT
    )
    formatter = (
        CustomJsonFormatter(format)
    )
    log_level = LOG_LEVEL

    def __init__(self):
        self.logger = LoggingMixin.get_logger(self.__class__.__name__)

    @classmethod
    def get_logger(cls, name):
        logger_ = logging.getLogger(name)
        handler = logging.StreamHandler()

        handler.setFormatter(cls.formatter)

        logger_.setLevel(cls.log_level)
        logger_.addHandler(handler)
        logger_.propagate = False
        return logger_


logger = LoggingMixin.get_logger(__name__)
